# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import pydantic

from .binding import Binding


class Policy(pydantic.BaseModel):
    bindings: list[Binding] = pydantic.Field(
        default=[],
        title="Bindings",
        description=(
            "Associates a list of `members`, or principals, with a `role`."
        )
    )


# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import fastapi
import pydantic

from .irequestprincipal import IRequestPrincipal


class NullRequestPrincipal(IRequestPrincipal, pydantic.BaseModel):

    @classmethod
    def depends(cls):
        return fastapi.Depends(cls)

    def is_anonymous(self) -> bool:
        return True

    def has_audience(self) -> bool:
        return False


# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from .iclient import IClient
from .objectidentifier import ObjectIdentifier


class ClientIdentifier(ObjectIdentifier[IClient]):
    openapi_title: str = 'Client ID'
    openapi_format: str = 'string'


# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import hashlib

from .objectidentifier import ObjectIdentifier
from .issuedaccesstoken import IssuedAccessToken


class IssuedAccessTokenIdentifier(ObjectIdentifier[IssuedAccessToken]):
    openapi_title: str = 'Token ID'
    openapi_format: str = 'string'

    @classmethod
    def parse_token(cls, token: str) -> 'IssuedAccessTokenIdentifier':
        return cls(hashlib.md5(str.encode(token, 'utf-8')).hexdigest())


# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from .authorizationexception import AuthorizationException


class MissingResponseType(AuthorizationException):
    __module__: str = 'cbra.ext.oauth2.types'
    error: str = 'invalid_request'
    message: str = "The 'response_type` parameter is required."